@@ -31,9 +31,13 @@ pub fn poll(
     // let mut i = 0;
     while count < iterations {
         let percent = count as f32 / iterations as f32;
-        
+
         // If count == 0, give 00... for remaining time as placeholder
-        let remaining_time_estimate = if count == 0 { Duration::new(0,0) } else { start.elapsed().div_f32(percent) };
+        let remaining_time_estimate = if count == 0 {
+            Duration::new(0, 0)
+        } else {
+            start.elapsed().div_f32(percent)
+        };
         print!(
             "\r{:20} ({:.2}%) {} / {}",
             count,
@@ -59,7 +63,8 @@ pub fn poll(
             thread::sleep(Duration::from_millis(poll_rate));
         }
 
-        count = offset + counters
+        count = offset
+            + counters
                 .iter()
                 .map(|c| c.load(Ordering::SeqCst))
                 .sum::<u32>();
